function M_hy = era_hy_mean(b,lambda,precision_of_num_integration,sigma)

% Exact Risk Analysis - computing mean for the hyperbolic thresholding rule
%
% M_hy = ERA_HY_MEAN(B,LAMBDA,PRECISION,SIGMA)
%
% Returns the mean for the hyperbolic thresholding rule computed
% for value(s) in B, for the threshold LAMBDA and for the
% gaussian noise standard deviation, SIGMA. SIGMA is an optional
% parameter; when omitted, M_hy is computed using SIGMA=1.
%
% PRECISION........(optional) Because the mean has to be computed numerically,
%                  this parameter must be set; if equal [] or not selected at all,
%                  then it is set defaultly

% (c)2004 Pavel Rajmic, Brno University of Technology, Czech Republic
% Last Revision 2004-08-23


precision_of_convergence = 10e-4; %dodelat 

if nargin<2 %!
    error('Too few input arguments.')
    
elseif nargin < 4  %default: computing for SIGMA=1
    
    integrand = inline('sqrt(x.^2-lambda^2) .* ( normpdf(x-b) - normpdf(x+b) )', 'x', 'lambda', 'b');
    
    if nargin == 2 %precision_of_num_integration was not specified
        precision_of_num_integration = [];
    end
    
    for cnt = 1:length(b) %we have to do it in cycle...
        sgn = sign(b(cnt)); %M_hy is an odd function, we'll utilize it
        M_hy(cnt) = sgn * integrate(integrand,lambda,precision_of_num_integration, precision_of_convergence, abs(b(cnt)));
    end
    
else  %user defined SIGMA
    M_hy = sigma * era_hy_mean(b./sigma,lambda/sigma,precision_of_num_integration);  %calling itself; formula for SIGMA<>1
end


%%%%% inner function %%%%%%
function integral = integrate(integrand, lower_limit, precision_of_num_integration, precision_of_convergence, b)
%computes numerically the integral from lambda to infinity.
%because infinity is problematic, we watch the integral's convergence while
%increasing the upper_limit

integral = 0;
prirustek_old = 0;  %finta, aby prosel prvni cyklus
lower = lower_limit; %now equal to lambda

%first part
upper = lower + 1;
prirustek_new = quad(integrand,lower,upper,precision_of_num_integration,[],lower_limit,b);
integral = integral + prirustek_new;
lower = upper;

%other parts
while (prirustek_new >= precision_of_convergence) | (prirustek_old <= prirustek_new)
    upper = lower + 1;
    prirustek_old = prirustek_new;
    prirustek_new = quad(integrand,lower,upper,precision_of_num_integration,[],lower_limit,b);
    integral = integral + prirustek_new;
    lower = upper;
end