function M_nng = era_nng_mean(b,lambda,iter_precision,sigma)

% Exact Risk Analysis - computing mean for the non-negative garrotte
%
% M_nng = ERA_NNG_MEAN(B, LAMBDA, ITER_PRECISION, SIGMA)
%
% Returns the mean for the non-negative garrotte thresholding rule computed
% for value(s) in B, for the threshold LAMBDA and for the
% gaussian noise standard deviation, SIGMA. SIGMA is an optional
% parameter; when omitted, M_nng is computed using SIGMA=1.
% ITER_PRECISION is a parameter defining the precision of iterative
% process (see remark below)
%
% Remark: We need to evaluate special integrals for computing the mean
% of non-negative garrotte. These can be only evaluated numerically.
% We use the iterative process as in Lemma 1 in:
% [1] Gao, H.-Y. (1998): Wavelet Shrinkage Denoising Using the Non-Negative
%     Garrote, J. Comput. Graph. Statist., 7(4), pp.469-488, 1998.

% (c)2003 Pavel Rajmic, Brno University of Technology, Czech Republic
% Last Revision 2004-08-23

if nargin<3 %!
    error('Too few input arguments.')
elseif nargin==3  %default: computing for SIGMA=1
    M_nng = era_hard_mean(b,lambda)-lambda^2*A_lambda_b(lambda,b,iter_precision);
else  %user defined SIGMA
    M_nng = sigma * era_nng_mean(b./sigma,lambda/sigma,iter_precision);  %calling itself; formula for SIGMA<>1
end


%%%%%%%%%%%%%%%%%%%%%%%% sub-functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [X,scitancu]=A_lambda_b(lambda,b,precision)
%function-the definite integral from [1], formula (2.2), computed iteratively
%and recursively according Lemma 1 in [1].
count=0;
gama(1)=sqrt(2*pi)*(1-normcdf(lambda));  %gama(0). Indexy musi byt posunuty,protoze Matlab neumi index 0
novy_scitanec=(b.^(2*count)*gama(count+1)) / factorial(2*count+1);
suma=novy_scitanec;  %nyni je totiz prvni scitanec zaroven sumou

%gama(2), ve znaceni Gao vsak gama(1):
gama(count+2)=lambda^(2*(count+1)-1)*exp(-lambda^2/2) + (2*(count+1)-1)*gama((count+2)-1);  %rekurzivni vypocet
while any(novy_scitanec > precision)  %precision tedy musi byt dosazena ve vsech slozkach
    count=count+1;
    novy_scitanec=(b.^(2*count)*gama(count+1)) / factorial(2*count+1);
    suma=suma+novy_scitanec;
    gama(count+2)=lambda^(2*(count+1)-1)*exp(-lambda^2/2) + (2*(count+1)-1)*gama((count+2)-1);  %rekurzivni vypocet
end

scitancu=count+1;
X=2.*b.*normpdf(b).*suma;
return