function R_nng=era_nng_risk(b,lambda,iter_precision,sigma)

% Exact Risk Analysis - computing risk for the non-negative garrotte
%
% R_nng = ERA_NNG_RISK(B, LAMBDA, ITER_PRECISION, SIGMA)
%
% Returns the L_2-risk for the non-negative garrotte thresholding rule computed
% for value(s) in B, for the threshold LAMBDA and for the
% gaussian noise standard deviation, SIGMA. SIGMA is an optional
% parameter; when omitted, R_nng is computed using SIGMA=1.
% ITER_PRECISION is a parameter defining the precision of iterative
% process (see remark below)
%
% Remark: We need to evaluate special integrals for computing the mean
% of non-negative garrotte. These can be only evaluated numerically.
% We use the iterative process as in Lemma 1 in:
% [1] Gao, H.-Y. (1998): Wavelet Shrinkage Denoising Using the Non-Negative
%     Garrote, J. Comput. Graph. Statist., 7(4), pp.469-488, 1998.

% (c)2003 Pavel Rajmic, Brno University of Technology, Czech Republic
% Last Revision 2004-08-23

if nargin<3 %!
    error('Too few input arguments.')
elseif nargin==3  %default: computing for SIGMA=1
    substitute = 2*lambda^2*(1-normcdf(lambda-b)+normcdf(-lambda-b)); %pomocny vektor (prip. skalar)
    R_nng = era_hard_risk(b,lambda) + 2*lambda^2*b.*A_lambda_b(lambda,b,iter_precision) +...
        lambda^4*B_lambda_b(lambda,b,iter_precision)- substitute;
else  %user defined SIGMA
    R_nng = sigma^2 * era_nng_risk(b./sigma,lambda/sigma,iter_precision);  %calling itself; formula for SIGMA<>1
end


%%%%%%%%%%%%%%%%%%%%%%%% sub-functions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [X,scitancu]=A_lambda_b(lambda,b,precision)
%function-the definite integral from [1], formula (2.2), computed iteratively
%and recursively according Lemma 1 in [1].
count=0;
gama(1)=sqrt(2*pi)*(1-normcdf(lambda));  %gama(0). Indexy musi byt posunuty,protoze Matlab neumi index 0
novy_scitanec=(b.^(2*count)*gama(count+1)) / factorial(2*count+1);
suma=novy_scitanec;  %nyni je totiz prvni scitanec zaroven sumou
%gama(2), ve znaceni Gao vsak gama(1):
gama(count+2)=lambda^(2*(count+1)-1)*exp(-lambda^2/2) + (2*(count+1)-1)*gama((count+2)-1);  %rekurzivni vypocet

while any(novy_scitanec > precision)  %precision tedy musi byt dosazena ve vsech slozkach
    count=count+1;
    novy_scitanec=(b.^(2*count)*gama(count+1)) / factorial(2*count+1);
    suma=suma+novy_scitanec;
    gama(count+2)=lambda^(2*(count+1)-1)*exp(-lambda^2/2) + (2*(count+1)-1)*gama((count+2)-1);  %rekurzivni vypocet
end

scitancu=count+1;
X=2.*b.*normpdf(b).*suma;
return


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [X,scitancu]=B_lambda_b(lambda,b,precision)
%function-the definite integral from [1], formula (2.3), computed iteratively
%and recursively according Lemma 1 in [1].
count=0;
gama(1)=sqrt(2*pi)*(1-normcdf(lambda));  %gama(0). Indexy musi byt posunuty,protoze Matlab neumi index 0
novy_scitanec=(b.^(2*count+2)*gama(count+1)) / factorial(2*count+2);
suma=novy_scitanec;  %nyni je totiz prvni scitanec zaroven sumou
%gama(2), ve znaceni Gao vsak gama(1):
gama(count+2)=lambda^(2*(count+1)-1)*exp(-lambda^2/2) + (2*(count+1)-1)*gama((count+2)-1);  %rekurzivni vypocet

while any(novy_scitanec > precision)  %precision tedy musi byt dosazena ve vsech slozkach
    count=count+1;
    novy_scitanec=(b.^(2*count)*gama(count+1)) / factorial(2*count+1);
    suma=suma+novy_scitanec;
    gama(count+2)=lambda^(2*(count+1)-1)*exp(-lambda^2/2) + (2*(count+1)-1)*gama((count+2)-1);  %rekurzivni vypocet
end

scitancu=count+1;
X=2*sqrt(2*pi)*normpdf(b)*((normpdf(lambda)/lambda-1+normcdf(lambda))) + 2*normpdf(b).*suma;
return