function varargout = mydwt(signal,decomp_depth,wav_name,vector_of_computations)

% MYDWT performs the wavelet decomposition up to a chosen depth, including multiresolution analysis (MRA)
%
% [coefs, mra_levels, mra_cumsum, L] = MYDWT(signal, decomp_depth, wav_name, vector_of_computations)
%
% Input parameters:
% ~~~~~~~~~~~~~~~~~
% signal...........input signal; vector
% decomp_depth.....decomposition depth; positive integer
% wav_name.........decomposition wavelet name; string, for example 'db3'
% vector_of_computations.....
%            ......logical vector of parameters, what has to be computed;
%                  its structure is [coefs, mra_levels, mra_cumsum], where
%                  we place 1, if we want to compute the respective outputs (decribed below),
%                  otherwise 0. If this vector is omitted, all three
%                  computations are done. It is also possible not to type
%                  the whole vector - then the missing elements are
%                  regarded as zeros.
%
% Output parameters:
% ~~~~~~~~~~~~~~~~~~
% The number and order of the outputs is determined by contents of input
% vector 'vector_of_computations'. L is always the last output parameter.
%
% coefs............wavelet coefficients of the input signal, variable type 'cell'.
%                  Structure: The first cell contains aproximation coefficients and in the subsequent there are
%                  the details stored, for example with decomp_depth=3, coefs={a3 d3 d2 d1}.
% mra_levels.......the contributions of the individual MR-analysis levels (one aprox. and several detail).
%                  Its type and structure is analogous to "coefs". The length of every cell is length(signal).
% mra_cumsum.......cumulative sum of these contributions (corresponds to projection onto the aproximation subspaces)
% L................vector giving the number of wavelet coefficients in individual levels;
%                  see 'wavedec.m' from Wavelet Toolbox of Matlab.

% (c)2003 Pavel Rajmic, Brno University of Technology, Czech Republic
% Last Revision 2004-08-23

%dwtmode('per');   %periodization  

if nargin<3
    error('Insufficient number of parameters!');
elseif nargin<4
    voc = [1 1 1];  %kdyz se 'voc' nezada, vypocita se vsechno (voc...vector of computations)
else
    voc = vector_of_computations; %pro zjednoduseni zapisu
    if ~isnumeric(voc), error('''vector_of_computations'' must be numeric!'), end
    if length(voc)>3, error('The length ''vector_of_computations'' should be 3 or less!'), end
    voc = ~(voc==0);  %prevod na logicky vektor nul a jednicek
    voc = [voc zeros(1,3-length(voc))];  %pripadna uprava na delku 3
end

% COEFS (pocitam vzdy, i kdyz se chce jen 'mra_levels' nebo 'mra_cumsum')
    [C,L] = wavedec(signal,decomp_depth,wav_name);  %hlavni vypocet nechavam na Wavelet Toolboxu
    %aproximacni a detailni koeficienty jako "cell array"; v poradi napr. a3,d3,d2,d1
    coefs=[{ C(1:L(1)) } detcoef(C,L,decomp_depth:-1:1,'cells')];

% MRA_LEVELS
if voc(2)
    %aproximacni level:
    pomocna = coefs{1}; %vezmu aproximacni koeficienty
    for count=1:decomp_depth  %musim se rekurzivne "vyfiltrovat" az na vychozi uroven
        pomocna=idwt(pomocna,[],wav_name, L(count+2) );  %pri inverzi vyjimam vzdy prostredni cast vysledku 
    end
    mra_levels{1}=pomocna;
    %detailni urovne:
    for count=2:(decomp_depth+1)
        pomocna = coefs{count};  %beru postupne detailni koeficienty
        for count2 = count:(decomp_depth+1)  %musim se rekurzivne "vyfiltrovat" az na vychozi uroven
            pomocna = idwt([],pomocna,wav_name, L(count2+1) );  %pri inverzi vyjimam vzdy prostredni cast vysledku
        end
        mra_levels{count}=pomocna;  %postupne naplnime bunky
    end
end
    
%MRA_CUMSUM
if voc(3)
    %musime napocitat postupne aproximace:
    for count=1:decomp_depth
        pomocna = appcoef(C,L,wav_name,decomp_depth-count+1);
        for count2=count:decomp_depth  %musim se rekurzivne "vyfiltrovat" az na vychozi uroven
            pomocna=idwt(pomocna,[],wav_name, L(count2+2) );
        end
        mra_cumsum{count}=pomocna;  %postupne naplnime bunky
        %!!! (toto asi neni nejrychlejsi, optimalni zpusob...) !!!
    end
    mra_cumsum{decomp_depth+1}=signal;  %jako posledni dostavam samozrejme puvodni signal
    %DORESIT NESROVNALOSTI V DELKACH!!!!!!
    
    %Tim jsme vytvorili bunecnou strukturu MRA paralelni k bunkam v "coefs",
    %ale v MRA jsou vsechny aproximace stejne dlouhe jako vychozi signal
end

% FORMATOVANI VYSTUPNICH ARGUMENTU:
count=1;
if voc(1)
    varargout(count)={coefs};
    count=count+1;
end
if voc(2)
    varargout(count)={mra_levels};
    count=count+1;
end
if voc(3)
    varargout(count)={mra_cumsum};
    count=count+1;
end
varargout(count)={L};  % tedy L je vzdy poslednim vystupem