function [figure_handle,handles_of_subplots] = plot_wav(signal,coefs,mra_levels,mra_cumsum)

% PLOT_WAV plots in a clear way the signal, its wavelet coefficients
% and the related multiresolution analysis (MRA)
%
% [figure_handle, handles_of_subplots] = PLOT_WAV(signal, coefs, mra_levels, mra_cumsum)
%
% Input parameters:
% ~~~~~~~~~~~~~~~~~
% The choice of the input parameters determines the plotting style. If the
% parameter is [] (empty matrix), its respective plot is omitted, otherwise
% its contents is plotted. It is not necessary to set the last few
% parameters, if their respective graphs should not be plotted.
%
% For example, command "plot_wav([],coeffs,MRA_levels)" produces the
% plot of wavelet coefficients and the related levels in MRA.
%
% signal..........signal
% coefs...........wavelet coefficients of the signal (as for the format, see 'MYDWT.M')
% mra_levels......contributions of the individual decomposition levels to the signal
%                 (as for the format, see 'MYDWT.M')
% mra_levels......cumulative contributions (see also 'MYDWT.M')
%
% Output parameters:
% ~~~~~~~~~~~~~~~~~~
% figure_handle.........handle of the main figure
% handles_of_subplots...matrix containing handles of all the subplots
 
% (c)2003 Pavel Rajmic, Brno University of Technology, Czech Republic
% Last Revision 2004-08-23


%Vyporadani se se vstupnimi argumenty:
if nargin<1, error('Insufficient number of parameters!'), end
if nargin<4, mra_cumsum=[]; end
if nargin<3, mra_levels=[]; end
if nargin<2, coefs=[]; end
if nargin>4, error('Too many input parameters!'), end
vop = ~[isempty(signal) isempty(coefs) isempty(mra_levels) isempty(mra_cumsum)];  %logicky vector_of_plots, trochu jiny nez v MYDWT.M


if ~any(vop)  %neni co kreslit
    error('At least one of the input parameters must be non-empty!')
    
elseif vop(1) & ~sum(vop(2:4)) %bude se kreslit jen signal samotny
    figure_handle = figure;  
    TitleSize=10;
    hos = plot(signal);  %je jasne, ze radku v grafu bude decomp_depth+2
    set(gca,'XLim',[1 length(signal)]);
    t=title(['Signal'],'FontSize',TitleSize,'VerticalAlignment','middle');
    %titulek je rozsiren o jmeno promenne, ktera nese signal, pokud takova je
    
else       %bude se neco kreslit i neco jineho krome samotneho signalu
    %zjistim hloubku dekompozice:
    if vop(2)
        decomp_depth = length(coefs)-1;
    elseif vop(3)
        decomp_depth = length(mra_levels)-1;
    else  %(jedna z techto moznosti jiste nastane)
        decomp_depth = length(mra_cumsum)-1;
    end
    
    sloupcu_celkem = sum(vop(2:4)); %kolik bude sloupcu v grafu
    sloupcu_zaplneno = 0;           %pomocna promenna kvuli kresleni subplotu
    
    figure_handle = figure;
    TitleSize=10;

    %hos...handles of subplots
    
    % SIGNAL
    if vop(1)  %pokud chceme vykreslit prubeh signalu
        hos(1,1) = subplot(decomp_depth+2,sloupcu_celkem,1); plot(signal);  %je jasne, ze radku v grafu bude decomp_depth+2
        hos(1,2:sloupcu_celkem) = 0;  %2:sloupcu_celkem muze byt prazdna mnozina..., pak se s 'hos' nic nestane
        set(gca,'XLim',[1 length(signal)]);
        t=title(['Signal'],'FontSize',TitleSize,'VerticalAlignment','middle');
        %titulek je rozsiren o jmeno promenne, ktera nese signal, pokud takova je
    end
      
    % COEFS 
    if vop(2)
        for count=1:decomp_depth   %detailni koeficienty
            hos(vop(1)+count,sloupcu_zaplneno+1) =...
                subplot(vop(1)+decomp_depth+1,sloupcu_celkem,prirad_cislo_grafu(count+vop(1),sloupcu_celkem,sloupcu_zaplneno));
            mystem(coefs{decomp_depth+2-count});
            set(gca,'XLim',[1 length(coefs{decomp_depth+2-count})]);
            if count==1;
                t=title(['Wavelet coefficients'],'FontSize',TitleSize,'VerticalAlignment','middle');
            end
            %postupne po bunkach; bunky cislovany v opacnem poradi nez subploty
        end
        
        %aprox. koeficienty:
        hos(vop(1)+decomp_depth+1,sloupcu_zaplneno+1) =...
            subplot( vop(1)+decomp_depth+1 , sloupcu_celkem , prirad_cislo_grafu(vop(1)+decomp_depth+1,sloupcu_celkem,sloupcu_zaplneno) );
        mystem(coefs{1}); 
        set(gca,'XLim',[1 length(coefs{1})]);
        
        sloupcu_zaplneno = sloupcu_zaplneno+1;  %zvedam pomocne pocitadlo
    end
    
    % MRA_LEVELS
    if vop(3)        
        for count=1:(decomp_depth+1) 
            hos(vop(1)+count,sloupcu_zaplneno+1) =...
                subplot(vop(1)+decomp_depth+1,sloupcu_celkem,prirad_cislo_grafu(count+vop(1),sloupcu_celkem,sloupcu_zaplneno));
            plot(mra_levels{decomp_depth+2-count});           
            set(gca,'XLim',[1 length(mra_levels{decomp_depth+2-count})]);
            %       set(gca,'YLim',[min(levels{2}) max(levels{2})]);  %jeste doresit
%             if sloupcu_zaplneno>0  %tzn. uz nejaky predchazejici je vlevo od tohoto aktualniho
%                 ylabel('<->','Rotation',0,'VerticalAlignment','middle')
%             end
            if count==1;
                t=title(['Multiresolution levels'],'FontSize',TitleSize,'VerticalAlignment','middle');
            end  
        end
                
        sloupcu_zaplneno = sloupcu_zaplneno+1;  %zvedam pomocne pocitadlo
    end    
    
    % MRA_CUMSUM
    if vop(4)
        if vop(1)  %pokud jsme kreslili signal
            meze_osy_y=get(hos(1,1),'YLim'); %zjistim meze osy y, abych s nimi mohl zobrazit vsechny
            %grafy v MRA (vhodne pro pozorovani vyvoje aproximaci)
        else  %musim najit nejvetsi a nejmensi cislo v 'mra_cumsum', abych prizpusobil meze u vsech urovni
            pomocny_vektor = [];
            for count=1:length(mra_cumsum)
                pomocny_vektor=[pomocny_vektor; mra_cumsum{count}]; %sloupec
            end
            meze_osy_y=[min(pomocny_vektor) max(pomocny_vektor)];
            clear pomocny_vektor
        end
        for count=1:decomp_depth   %detailni koeficienty
            hos(vop(1)+count,sloupcu_zaplneno+1) =...
                subplot(vop(1)+decomp_depth+1,sloupcu_celkem,prirad_cislo_grafu(count+vop(1),sloupcu_celkem,sloupcu_zaplneno));
            plot(mra_cumsum{decomp_depth+2-count});
            set(gca,'XLim',[1 length(mra_cumsum{decomp_depth+2-count})]);
            set(gca,'YLim',meze_osy_y);
%             if sloupcu_zaplneno>0  %tzn. uz nejaky predchazejici je vlevo od tohoto aktualniho
%                 ylabel('<->','Rotation',0,'VerticalAlignment','middle')
%             end            
            if count==1;
                t=title(['Multiresolution analysis'],'FontSize',TitleSize,'VerticalAlignment','middle');
            end
        end

        hos(vop(1)+decomp_depth+1,sloupcu_zaplneno+1) =...
            subplot( vop(1)+decomp_depth+1 , sloupcu_celkem , prirad_cislo_grafu(vop(1)+decomp_depth+1,sloupcu_celkem,sloupcu_zaplneno) );
        plot(mra_cumsum{1}); %prvni, nejhrubsi aproximace
        set(gca,'XLim',[1 length(mra_cumsum{1})]);
        set(gca,'YLim',meze_osy_y);
%         if sloupcu_zaplneno>0  %tzn. uz nejaky predchazejici je vlevo od tohoto aktualniho
%             ylabel('<->','Rotation',0,'VerticalAlignment','middle')
%         end
                
        %zde uz nemusim zvedat pomocne pocitadlo
    end
    
    % LABELS ETC.
    for count=1:decomp_depth   %detailni koeficienty
        set( get(hos(vop(1)+count,1),'YLabel') , 'String' , ['d' num2str(count)])
        %         set( hos(vop(1)+count,1) , ) =...
        %             subplot(vop(1)+decomp_depth+1,sloupcu_celkem,prirad_cislo_grafu(count+vop(1),sloupcu_celkem,sloupcu_zaplneno));
        %         plot(mra_cumsum{decomp_depth+2-count});
        %         set(gca,'XLim',[1 length(mra_cumsum{decomp_depth+2-count})]);
        %         set(gca,'YLim',meze_osy_y);
    end    
    %aproximacni koeficienty:
    set( get(hos(vop(1)+decomp_depth+1,1),'YLabel') , 'String' , ['a' num2str(decomp_depth)])

end

handles_of_subplots=hos;

%%%%%%%%%%%% Vnitrni pomocna funkce %%%%%%%%%%%%%
function y=prirad_cislo_grafu(count,sloupcu_celkem,sloupcu_zaplnenych)
%pomocna funkce, umoznujici napsat vyse uvedeny program zaroven
%pro pripad vicesloupcoveho obrazku
y=sloupcu_celkem*(count-1)+1+sloupcu_zaplnenych;