function [signal_thresh,mse,N_thresh,coefs_thresh,L,coefs_orig]=...
    thresh(signal_orig,decomp_depth,wav_name,thresh_rule,thresh_params,plot_boolean)

% THRESH performs the threshoding of the wavelet coefficients and returns the modified signal
%
% [SIGNAL_THRESH, MSE, N_THRESH, COEFS_THRESH, L, COEFS_ORIG] =
%      THRESH( SIGNAL_ORIG, DECOMP_DEPTH, WAV_NAME, THRESH_RULE, THRESH_PARAMS, PLOT_BOOLEAN)
%
% Input parameters:
% ~~~~~~~~~~~~~~~~~
% SIGNAL_ORIG.....input signal, which is to be thresholded; column vector
% DECOMP_DEPTH....depth of wavelet decomposition
% WAV_NAME........name of the decomposition wavelet, for example 'db3'
% THRESH_RULE.....abbreviation of the thresholding rule to be used; string
%                  'h'.....hard thresholding
%                  's'.....soft thresholding
%                  'ss'....semisoft thresholding
%                  'nng'...non-negative garrotte
%                  'hy'....hyperbolic thresholding
% THRESH_PARAMS...thresholding parameters; variable of type 'cell'; its structure 
%                 differs in dependence on chosen thresholding rule
%             - for 'h','s','nng','hy' is the structure: {threshold_value depth_of_thresholding} (cell array).
%                    Variable "depth_of_thresholding" is optional.
%                    Allowed alternatives:
%                    ** "threshold_value"...single value & "depth_of_thresholding"...not given or equal to 1
%                       => leads to thresholding just the finest detail coefficients
%                          with threshold "threshold_value",
%                    ** "threshold_value"...single value & "depth_of_thresholding"...number > 1
%                       => leads to thresholding detail coeficients up to depth
%                          "depth_of_thresholding" with threshold "threshold_value",
%                    ** "threshold_value"...vector of length >1
%                       => leads to not considering "depth_of_thresholding", even if stated,
%                          and to thresholding length(threshold_value) levels of detail coefficients.
%                          The order of values in "threshold_value" corresponds to the order
%                          of levels in wavelet decomposition (see 'MYDWT.M').
%             - for 'ss' is the structure: {threshold1_value threshold2_value depth_of_thresholding} (cell array).
%                    Its allowed alternatives and meaning is the same as above. The variable
%                    "depth_of_thresholding" is again optional.
%           NOTE! The last value in "threshold_value" corresponds with the finest detail coefficients etc.
% PLOT_BOOLEAN....optional logical parameter; in case it contains a non-zero value,
%                 it leads to producing a clear figure with signal plot, plot of its wavelet coefficients in
%                 individual levels with marking the threshold values. If PLOT_BOOLEAN=0
%                 no figure is produced.
%
% Output parameters:
% ~~~~~~~~~~~~~~~~~~
% SIGNAL_THRESH...resultant signal after thresholding the wavelet coefficients
% MSE.............estimate of the mean square error:
%                      norm(SIGNAL_THRESH-SIGNAL_ORIG)/length(SIGNAL_ORIG)
% N_THRESH........vector, which states how many coefficients in individual detail levels were set to zero.
%                 The order of the elements reflects the order in "threshold_value".
% COEFS_THRESH....thresholded wavelet coefficients;
%                 structure is identical to the structure of output from 'MYDWT.M'
% L...............vector giving the number of wavelet coeffs in individual levels;
%                 se 'MYDWT.M'
% COEFS_ORIG......wavelet coefficients of the original signal; structure identical to COEFS_THRESH.

% (c)2003 Pavel Rajmic, Brno University of Technology, Czech Republic
% Last Revision 2004-08-23


% POZNAMKA:
% Procedura ma 3 kroky:
% 1.transformace do waveletove domeny; 2.prahovani; 3.inverzni transformace

%close all

%Uvodni kontrola a rozlisovani hloubky prahovani + jestli na kazdou uroven
%se ma pouzit jiny prah apod.
%=========================================================================
if nargin<5; error('Insufficient number of parameters!'); end
if nargin==5, plot_boolean=0; end

%Rozliseni zvoleneho prahovaciho pravidla
%========================================
if ~ischar(thresh_rule)
    error('THRESH_RULE must be string, see help.')
end
%%%%%%%%%%%%%%%
if ismember({thresh_rule},{'h' 's' 'hy' 'nng'})
    if length(thresh_params{1})==1
        if length(thresh_params)==1  %jestlize nebyla zadana hloubka prahovani
            thresh_params{2}=1;
        elseif...          %tedy hloubka byla zadana
        length(thresh_params{2})~=1 | ~isnumeric(thresh_params{2}) | thresh_params{2}==0
            %(kontrola)
            error('The depth of thresholding, ''depth_of_thresholding'' must be integer 1 <= depth_of_thresholding <= DECOMP_DEPTH')
        end
        % Konstrukce vektoru prahu prislusnych k detailnim urovnim:
        thresh_params{1}=thresh_params{1}*ones(1,thresh_params{2});
    elseif length(thresh_params{1})>decomp_depth %tedy threshold je vektor, kontrola delky:
        error('You must arrange 1 <= length(threshold_value) <= DECOMP_DEPTH !')
    end
    %hloubku prahovani jiz nepotrebuji; nyni je obsazena ve vektoru prahu:
    thresh_params=thresh_params(1);
%%%%%%%%%%%%%%    
elseif ismember({thresh_rule},{'ss'})
    if length(thresh_params)<2
        error('When using semisoft thresholding, you must set two thresholds (alternatively two vectors)!')
    end
    if length(thresh_params{1})~=length(thresh_params{2})
        error('The lengths of the two threshold vectors must agree when using semisoft thresholing!')
    end
    if any(thresh_params{2} < thresh_params{1})
        error('The second vector of thresholds must be greater (or equal) in every element!')
    end
    if length(thresh_params{1})==1   %prah je cislo
        if length(thresh_params)==2  %jestlize nebyla zadana hloubka prahovani
            thresh_params{3}=1;
        elseif...          %tedy hloubka byla zadana
        length(thresh_params{3})~=1 | ~isnumeric(thresh_params{3})...
        | thresh_params{3}==0 | thresh_params{3}>decomp_depth %kontrola
            error('The depth of thresholding, ''depth_of_thresholding'' must be integer 1 <= depth_of_thresholding <= DECOMP_DEPTH')
        end
        % Konstrukce vektoru prahu prislusnych k detailnim urovnim:
        thresh_params{1}=thresh_params{1}*ones(1,thresh_params{3});
        thresh_params{2}=thresh_params{2}*ones(1,thresh_params{3});
    elseif length(thresh_params{1})>decomp_depth %tedy threshold je vektor, kontrola delky:
        error('You must arrange 1 <= length(threshold_value) <= DECOMP_DEPTH !')    
    end
    %hloubku prahovani jiz nepotrebuji; nyni je obsazena ve vektorech prahu:
    thresh_params=thresh_params(1:2);
%%%%%%%%%%%%%    
%elseif ismember({thresh_rule},{'ndg'})  %%priprava na n degree garrotte
    
else error('Incorrect abbreviation of the thresholding rule!')
end

%ad 1.
%=====
[coefs,L]=mydwt(signal_orig,decomp_depth,wav_name,[1 0 0]);  %rozklad bez obrazku

%ad 2.
%=====
thresh_depth=length(thresh_params{1});  %hloubka prahovani

%Tvrde prahovani
if ismember({thresh_rule},{'h'})
    threshold=thresh_params{1};
    for count= (decomp_depth+1):-1:(decomp_depth+2-thresh_depth)
        count2=thresh_depth-(decomp_depth+1)+count;
        podprahem = abs(coefs{count})<=threshold(count2); %logicky vektor (sloupec)
        coefs_thresh{count2}(podprahem)=0;                %vynulovani podprahovych hodnot
        %nadprahove koeficienty neni treba u tvrdeho prahovani upravovat:
        coefs_thresh{count2}(~podprahem)=coefs{count}(~podprahem);
        %musi byt sloupce aby dobre fungovala inverze:
        coefs_thresh{count2}=coefs_thresh{count2}';
        N_thresh(count2)=sum(podprahem); %pocet wav. koeficientu, ktere byly vynulovany
    end
        
%Mekke, hyperbolicke a nezaporna garota
elseif ismember({thresh_rule},{'s' 'hy' 'nng'})
    switch thresh_rule
        case 's',  fhandle=@thresh_s;  %soft
        case 'hy', fhandle=@thresh_hy; %hyperbolic
        case 'nng',fhandle=@thresh_nng; %non-negative garrotte
    end
    [coefs_thresh,N_thresh]=...
        cycle(fhandle,thresh_params{1},coefs((decomp_depth+2-thresh_depth):(decomp_depth+1)));
    
%Polomekke prahovani
elseif ismember({thresh_rule},{'ss'})
    threshold=thresh_params{1};
    for count= (decomp_depth+1):-1:(decomp_depth+2-thresh_depth)
        count2=thresh_depth-(decomp_depth+1)+count;
        podprahem = abs(coefs{count})<=threshold(count2);  %logicky vektor (sloupec)
        coefs_thresh{count2}(podprahem)=0;                 %vynulovani podprahovych hodnot
        %uprava nadprahovych podle zvoleneho prah. pravidla:
        coefs_thresh{count2}(~podprahem)=...
            thresh_ss(coefs{count}(~podprahem),thresh_params{1}(count2),thresh_params{2}(count2));
        %musi byt sloupce aby dobre fungovala inverze:
        coefs_thresh{count2}=coefs_thresh{count2}';
        N_thresh(count2)=sum(podprahem);
    end
end

% pridam k vyprahovanym aproximacni a neprahovane detailni a tim
% ziskam kompletni upravene waveletove koeficienty pripravene k inverzi:
coefs_thresh=[coefs(1:(decomp_depth+1-thresh_depth)) coefs_thresh];


%ad 3.
%=====
signal_thresh=myidwt(coefs_thresh,L,wav_name)';  %zpetna transformace bez grafu; sloupec
coefs_orig=coefs;
mse=norm(signal_thresh-signal_orig) / length(signal_orig);
%%% Konec vypoctove casti %%%

%%%%%%% KRESLICI CAST %%%%%%%
if ~plot_boolean, return, end
TitleSize=10;
%barvy figurujici v grafech:
Color_killed='r'; Color_retained='b'; Color_thresholds='m'; Type_thresholds=':';
f=figure;
%puvodni signal:
subplot(decomp_depth+3,1,1); plot(signal_orig);
set(gca,'XLim',[1 length(signal_orig)])
t=title('Original signal','FontSize',TitleSize);
axis tight
%prahovany signal:
subplot(decomp_depth+3,1,2); plot(signal_thresh);
set(gca,'XLim',[1 length(signal_thresh)])
t=title('Signal after thresholding','FontSize',TitleSize);
axis tight
%detailni koeficienty prahovane
threshold=thresh_params{1};
if ismember({thresh_rule},{'ss'})
    threshold2_ss=thresh_params{2};   %nova promenna, ktera nam umozni pro polomekke prahovani
end                                   %naznacit i uroven druheho prahu 
thresh_depth=length(threshold);
for count= (decomp_depth+1):-1:(decomp_depth+2-thresh_depth)
    count2=thresh_depth-decomp_depth+count-1;
    delka=length(coefs{count});
    podprahem = abs(coefs{count})<=threshold(count2);  %logicky vektor (sloupec)
    subplot(decomp_depth+3,1,decomp_depth+4-count);
    pomocna=(1:delka)';  %musi byt sloupec, nebot wav. koeficienty v bunkach jsou sloupce
    mystem(pomocna(podprahem),coefs{count}(podprahem),Color_killed); %podprahove na prislusnych mistech
    hold on
    %nadprahove na prislusnych mistech:
    mystem(pomocna(~podprahem),coefs{count}(~podprahem),Color_retained);
    set(gca,'XLim',[1 delka]);
    %naznaceni prahu:
    plot([1 delka],[threshold(count2) -threshold(count2);threshold(count2) -threshold(count2)],...
        [Color_thresholds Type_thresholds],'LineWidth',.5)
    if ismember({thresh_rule},{'ss'})  %pro polomekke prahovani naznacime jeste druhy prah
       plot([1 delka],[threshold2_ss(count2) -threshold2_ss(count2);threshold2_ss(count2) -threshold2_ss(count2)],...
           [Color_thresholds Type_thresholds],'LineWidth',.5)
    end
    %set(gca,'XLim',[1 delka])
    ylabel(['d' num2str(decomp_depth-count+2)],'FontWeight','bold');
    axis tight
end
%detailni koeficienty neprahovane
for count= (decomp_depth+1-thresh_depth):-1:2
    subplot(decomp_depth+3,1,decomp_depth+4-count);
    mystem(coefs{count},Color_retained);
    hold on
    set(gca,'XLim',[1 length(coefs{count})])
    ylabel(['d' num2str(decomp_depth-count+2)]);
    axis tight
end

%aproximacni koeficienty
subplot(decomp_depth+3,1,decomp_depth+3); mystem(coefs{1});
set(gca,'XLim',[1 length(coefs{1})])
ylabel(['a' num2str(decomp_depth)]);
axis tight


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Vnitrni funkce %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [coefs_thresh,N_thresh]=cycle(function_handle,threshold,coefs_to_threshold)
% Provede prahovani v cyklu "for" pro jednotlive detailni koeficienty a pro
% zadany druh prahovani; Udelano pro 's','hy','nng'
thresh_depth=length(coefs_to_threshold);
for count= 1:thresh_depth
    podprahem = abs(coefs_to_threshold{count})<=threshold(count);  %logicky vektor (sloupec)
    coefs_thresh{count}(podprahem)=0;                  %vynulovani podprahovych hodnot
    %uprava nadprahovych podle zvoleneho prah. pravidla:
    coefs_thresh{count}(~podprahem)=...
        feval(function_handle,coefs_to_threshold{count}(~podprahem),threshold(count));
    coefs_thresh{count}=coefs_thresh{count}';  %musi byt sloupce aby dobre fungovala inverze
    N_thresh(count)=sum(podprahem);
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Y=thresh_h(X)
% Tvrde prahovani
Y=X;
return

function Y=thresh_s(X,lambda)
% Mekke prahovani
Y=X-sign(X)*lambda;  %mala finta
return

function Y=thresh_hy(X,lambda)
% Hyperbolicke prahovani
Y=sign(X).*sqrt(X.^2-lambda^2);
return

function Y=thresh_nng(X,lambda)
% Non-negative garrotte
Y=X-(lambda^2./X);
return

function Y=thresh_ss(X,lambda1,lambda2)
% Polomekke prahovani
if lambda1<lambda2  %jinak nastava rovnost a pujde o tvrde
                    %prahovani v tomto meritku (plyne z bodove limity lambda2->lambda1)
    podlambda2= abs(X)<=lambda2;  %logicky vektor
    X(podlambda2)=(lambda2/(lambda2-lambda1))*sign(X(podlambda2)).*(abs(X(podlambda2))-lambda1);
end
%ostatni nechavame stejne
Y=X;
return